/** @file eeprom.c
 
   This file is taken from the Windows driver and adapted for Linux.
 
   @par CREATION  
   @author Krauth Julien
   @date   09.10.2006
   
   @par VERSION
   @verbatim
   $LastChangedRevision:$
   $LastChangedDate:$
   @endverbatim   
   
   @par LICENCE
   @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
        
        ADDI-DATA GmbH 
        Dieselstrasse 3 
        D-77833 Ottersweier 
        Tel: +19(0)7223/9493-0 
        Fax: +49(0)7223/9493-92 
        http://www.addi-data-com 
        info@addi-data.com         
   @endverbatim   
 */
 
/*  +-----------------------------------------------------------------------+  */
/*  | (C) ADDI-DATA GmbH          Dieselstrasse 3      D-77833 Ottersweier  |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Tel : +49 (0) 7223/9493-0     | email    : info@addi-data.com         |  */
/*  | Fax : +49 (0) 7223/9493-92    | Internet : http:$/$/www.addi-data.com   |*/
/*  +-----------------------------------------------------------------------+  */
/*  | Project   : -                 | Compiler : BORLAND C                  |  */
/*  | Modulname : Eeprom.c          | Version  : 3.1                        |  */
/*  +-------------------------------+---------------------------------------+  */
/*  | Author    : S.Weber           | Date     : 26.02.04                   |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Description : Eeprom management                                       |  */
/*  +-----------------------------------------------------------------------+  */
/*  |                             UPDATE'S                                  |  */
/*  +-----------------------------------------------------------------------+  */
/*  |   Date   |   Author  |          Description of updates                |  */
/*  +----------+-----------+------------------------------------------------+  */
/*  |          |           |                                                |  */
/*  +-----------------------------------------------------------------------+  */
/* Include*/
/*+--------------------------------------------------------------------------------------------+*/
/*| INCLUDE                                                                                    |*/
/*+--------------------------------------------------------------------------------------------+*/



#include "../apci3701-private.h"

#include "include/pcidetection.h"
#include "include/typedef.h"
#include "include/prototype.h"
#include "include/in_out.h"

static int check_EepromRead(pstr_AddressInformation ps_Address)
{
	return ( (INPDW (ps_Address, 0) & 1) == 0 ) ;
}

/*i_EepromRead*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : INT  i_EepromRead    (pstr_AddressInformation ps_Address,   |*/
/*|                                           WORD                     w_offset,    |*/
/*|                                           PWORD                   pw_Value)     |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function read a value from the EEPROM 93C76.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address : PCI eeprom base address                        |*/
/*|                     w_offset   : Offset of the adress to read                   |*/
/*|                     pw_Value   : PCI eeprom 16 bit read value.                  |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
INT  i_EepromRead    (pstr_AddressInformation ps_Address,
                      WORD                     w_offset,
                      WORD *                  pw_Value)
        {
        DWORD  dw_RegisterValue     = 0;
        
        /*******************************/
        /* Set the eeprom read address */
        /*******************************/
        dw_RegisterValue = w_offset;
        dw_RegisterValue = dw_RegisterValue << 16;
        OUTPDW (ps_Address , 4, dw_RegisterValue);
        
        /************************/
        /* Set the read command */
        /************************/
        OUTPDW (ps_Address, 0, 0x8);
                
        /* timeout 3 seconds */          
       	if ( apci3701_eeprom_busywait_with_timeout(check_EepromRead,ps_Address,30000) )
       		return -1;

		dw_RegisterValue = INPDW (ps_Address, 4);
           
		*pw_Value = (unsigned short) (dw_RegisterValue & 0xFFFF);
           
        return (0);
        
        }
        
static int check_EepromWrite(pstr_AddressInformation ps_Address)
{
	return ( (INPDW (ps_Address, 0) & 1) == 0 ) ;
}
        	        
/*i_EepromWrite*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : INT  i_EepromWrite(pstr_AddressInformation ps_Address,      |*/
/*|                                        WORD                     w_offset,       |*/
/*|                                        WORD                     w_EepromValue)  |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function read a value from the EEPROM 93C76.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address    : PCI eeprom base address                     |*/
/*|                     w_offset      : Offset of the adress to read                |*/
/*|                     w_EepromValue : PCI eeprom 16 bit value to write.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
INT  i_EepromWrite (pstr_AddressInformation ps_Address,
                    WORD                     w_offset,
                    WORD                     w_EepromValue)
        {
        DWORD  dw_RegisterValue = 0;

        /******************************************/
        /* Set the eeprom write address and value */
        /******************************************/
        dw_RegisterValue = w_offset;
        dw_RegisterValue = dw_RegisterValue << 16;
        dw_RegisterValue = dw_RegisterValue | w_EepromValue;
        OUTPDW (ps_Address, 4, dw_RegisterValue);
        
        /*************************/
        /* Set the write command */
        /*************************/
        OUTPDW (ps_Address, 0, 0x18);

        /* timeout 3 seconds */   	
        if ( apci3701_eeprom_busywait_with_timeout(check_EepromWrite,ps_Address,30000) )
        	return -1;
       
		return (0);
        
        }
/*i_EepromReadBlock*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : INT  i_EepromReadBlock                                      |*/
/*                                    (pstr_AddressInformation ps_Address,          |*/
/*|                                    WORD                     w_StartAddress,     |*/
/*|                                    WORD                     w_Size,             |*/
/*|                                    PWORD                   pw_Value)            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function read a block from the EEPROM 93C76.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address       : PCI eeprom base address                  |*/
/*|                     w_StartAddress   : Start address                            |*/
/*|                     pw_Value         : PCI eeprom 16 bit read block.            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
INT  i_EepromReadBlock  (pstr_AddressInformation ps_Address,
                         WORD                     w_StartAddress,
                         WORD                     w_Size,
                         PWORD                   pw_Value)
        {
        /************************/
        /* Variable declaration */
        /************************/
        INT  i_Error           = 0;
        WORD w_AddressCounter  = 0;
        
        for (w_AddressCounter = 0;
             (w_AddressCounter < w_StartAddress) && (i_Error == 0);
             w_AddressCounter ++, pw_Value ++)
           {
           /*******************/
           /* Read the Eeprom */
           /*******************/
           i_Error = i_EepromRead (ps_Address, (WORD) (w_StartAddress + w_AddressCounter), pw_Value);
        
           }
        return (i_Error);

        }
/*i_EepromWriteBlock*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : INT  i_EepromWriteBlock                                     |*/
/*                                    (pstr_AddressInformation ps_Address,          |*/
/*|                                    WORD                     w_StartAddress,     |*/
/*|                                    WORD                     w_Size,             |*/
/*|                                    PWORD                   pw_Value)            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function write a block to the EEPROM 93C76.            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address       : PCI eeprom base address                  |*/
/*|                     w_StartAddress   : Start address                            |*/
/*|                     pw_Value         : PCI eeprom 16 bit write block.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
INT  i_EepromWriteBlock  (pstr_AddressInformation ps_Address,
                          WORD                     w_StartAddress,
                          WORD                     w_Size,
                          PWORD                   pw_Value)
        {
        /************************/
        /* Variable declaration */
        /************************/
        INT  i_Error           = 0;
        WORD w_AddressCounter  = 0;
        
        for (w_AddressCounter = 0;
             (w_AddressCounter < w_StartAddress) && (i_Error == 0);
             w_AddressCounter ++, pw_Value ++)
           {
           /********************/
           /* Write the Eeprom */
           /********************/
           i_Error = i_EepromWrite (ps_Address, (WORD) (w_StartAddress + w_AddressCounter), *pw_Value);
        
           }
        return (i_Error);

        }
