/** @file header.c
 
   This file is taken from the Windows driver and adapted for Linux.
 
   @par CREATION  
   @author Krauth Julien
   @date   09.10.2006
   
   @par VERSION
   @verbatim
   $LastChangedRevision:$
   $LastChangedDate:$
   @endverbatim   
   
   @par LICENCE
   @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
        
        ADDI-DATA GmbH 
        Dieselstrasse 3 
        D-77833 Ottersweier 
        Tel: +19(0)7223/9493-0 
        Fax: +49(0)7223/9493-92 
        http://www.addi-data-com 
        info@addi-data.com         
   @endverbatim   
 */
 
/*  +-----------------------------------------------------------------------+  */
/*  | (C) ADDI-DATA GmbH          Dieselstrasse 3      D-77833 Ottersweier  |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Tel : +49 (0) 7223/9493-0     | email    : info@addi-data.com         |  */
/*  | Fax : +49 (0) 7223/9493-92    | Internet : http:$/$/www.addi-data.com   |*/
/*  +-----------------------------------------------------------------------+  */
/*  | Project   : -                         | Compiler : BORLAND C          |  */
/*  | Modulname : UpdateHeaderAndFirmware.c | Version  : 3.1                |  */
/*  +-------------------------------+---------------------------------------+  */
/*  | Author    : S.Weber           | Date     : 05.03.04                   |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Description : Header and firmware update functions                    |  */
/*  +-----------------------------------------------------------------------+  */
/*  |                             UPDATE'S                                  |  */
/*  +-----------------------------------------------------------------------+  */
/*  |   Date   |   Author  |          Description of updates                |  */
/*  +----------+-----------+------------------------------------------------+  */
/*  |          |           |                                                |  */
/*  +-----------------------------------------------------------------------+  */

#include "../apci3701-private.h"

#include "include/pcidetection.h"
#include "include/typedef.h"
#include "include/prototype.h"

/*i_UpdateHeader_GetHeaderSize*/
/*+----------------------------------------------------------------------------+*/
/*| Function Name     : INT i_UpdateHeader_GetHeaderSize                       |*/
/*|                         (pstr_AddressInformation          ps_Address,      |*/
/*|                          WORD *                           pw_HeaderSize)   |*/
/*|                          UpdateHeaderAndFirmwareCallBack   v_InfoCallBack) |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Get the header size                                    |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address     : Board address                         |*/
/*|                     v_InfoCallBack : Info call back function               |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : pw_HeaderSize : Header syze (Byte)                     |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      :  0: No error                                           |*/
/*|                     -1: Error                                              |*/
/*|                      1: Warning                                            |*/
/*+----------------------------------------------------------------------------+*/
INT i_UpdateHeader_GetHeaderSize (pstr_AddressInformation          ps_Address,
                                  WORD *                           pw_HeaderSize)
        {
        /************************/
        /* Variable declaration */
        /************************/
        INT  i_EepromRet   = 0;
        
        *pw_HeaderSize = 0;
        
        /***********************/
        /* Get the header size */
        /***********************/
        i_EepromRet = i_EepromRead (ps_Address,
                                    264 / 2,
                                    pw_HeaderSize);
        
        /* Error management */
        /***********************/
        /* Test if error occur */
        /***********************/
        if (i_EepromRet != 0)
           {
           printk ("Read Eeprom address 264 error. Error = %d", i_EepromRet);
           return (-1);        
           }
        /**************************************************/
        /* Add the main header size + the  boot load size */
        /**************************************************/
        *pw_HeaderSize = *pw_HeaderSize + 10;
        
        return (0);

        }
/*i_UpdateHeader_GetEepromSize*/
/*+----------------------------------------------------------------------------+*/
/*| Function Name     : INT i_UpdateHeader_GetEepromSize                       |*/
/*|                         (pstr_AddressInformation          ps_Address,      |*/
/*|                          WORD *                           pw_HeaderSize)   |*/
/*|                          UpdateHeaderAndFirmwareCallBack   v_InfoCallBack) |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Get the header size                                    |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address     : Board address                         |*/
/*|                     v_InfoCallBack : Info call back function               |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : pw_HeaderSize : Header syze (Byte)                     |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      :  0: No error                                           |*/
/*|                     -1: Error                                              |*/
/*|                      1: Warning                                            |*/
/*+----------------------------------------------------------------------------+*/
INT i_UpdateHeader_GetEepromSize (pstr_AddressInformation          ps_Address,
                                  WORD *                           pw_EepromSize)
        {
        /************************/
        /* Variable declaration */
        /************************/
        INT  i_EepromRet   = 0;
        
        *pw_EepromSize = 0;
        
        /***********************/
        /* Get the header size */
        /***********************/
        i_EepromRet = i_EepromRead (ps_Address,
                                    264 / 2,
                                    pw_EepromSize);
        
        /* Error management */
        /***********************/
        /* Test if error occur */
        /***********************/
        if (i_EepromRet != 0)
           {
              printk  ("Read Eeprom address 264 error. Error = %d", i_EepromRet);
           return (-1);
        
           }
        /**************************************************/
        /* Add the main header size + the  boot load size */
        /**************************************************/
        *pw_EepromSize = *pw_EepromSize + 10 + 256;
        
        return (0);

        }

/*i_UpdateHeader_ReadBoardEeprom*/
/*+----------------------------------------------------------------------------+*/
/*| Function Name     : INT i_UpdateHeader_ReadBoardEeprom                     |*/
/*|                        (pstr_AddressInformation          ps_Address,       |*/
/*|                         WORD                              w_StartAddress,  |*/
/*|                         WORD                              w_SizeToRead,    |*/
/*|                         WORD*                            pw_Eeprom,        |*/
/*|                         UpdateHeaderAndFirmwareCallBack   v_InfoCallBack)  |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Read the selected Eeprom block                         |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address     : Board Address                         |*/
/*|                     w_StartAddress : Eeprom start address                  |*/
/*|                     w_SizeToRead   : Size to read                          |*/
/*|                     v_InfoCallBack : Info call back function               |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : pw_Eeprom : Eeprom data                                |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      :  0: No error                                           |*/
/*|                     -1: Error                                              |*/
/*|                      1: Warning                                            |*/
/*+----------------------------------------------------------------------------+*/
INT i_UpdateHeader_ReadBoardEeprom (pstr_AddressInformation          ps_Address,
                                    WORD                              w_StartAddress,
                                    WORD                              w_SizeToRead,
                                    WORD*                            pw_Eeprom)
        {
        /************************/
        /* Variable declaration */
        /************************/
        INT     i_EepromRet      = 0;
        WORD    w_AddressCounter = 0;
        
        /***************************/
        /* Varaible initialisation */
        /***************************/
        w_StartAddress = w_StartAddress / 2;
        
        for (w_AddressCounter = 0;
             (w_AddressCounter < (w_SizeToRead / 2)) && (i_EepromRet >= 0);
             w_AddressCounter ++, pw_Eeprom ++)
           {
           /***********************/
           /* Get the header size */
           /***********************/
           i_EepromRet = i_EepromRead (ps_Address,
                                       (WORD) (w_StartAddress + w_AddressCounter),
                                       pw_Eeprom);
           
           /* Error management */
           /***********************/
           /* Test if error occur */
           /***********************/
           if (i_EepromRet < 0)
              {
              printk ("Read Eeprom address %d error. Error = %d", (w_StartAddress + w_AddressCounter) / 2, i_EepromRet);            
              return (-1);
              }
                 
           }
        return (i_EepromRet);

        }
        
/*v_UpdateHeader_GetHeaderInfos*/
/*+----------------------------------------------------------------------------+ */
/*| Function   Name   : VOID    v_UpdateHeader_GetHeaderInfos                  | */
/*|                            (WORD* pw_HeaderData,                           | */
/*|                             WORD* pw_GlobalHeaderSizeAddress,              | */
/*|                             WORD* pw_MainAnalogInputStartAddress,          | */
/*|                             WORD* pw_LVDTHeaderStartAddress,               | */
/*|                             WORD* pw_TimeCounterHeaderStartAddress)        | */
/*+----------------------------------------------------------------------------+ */
/*| Task              : Return header address                                  | */
/*+----------------------------------------------------------------------------+ */
/*| Input Parameters  : pw_HeaderData : Board header (Offset 0 = start address | */
/*|                                     0x100)                                 | */
/*+----------------------------------------------------------------------------+ */
/*| Output Parameters : pw_GlobalHeaderSizeAddress        : Main header size   | */
/*|                                                         address            | */
/*|                     pw_MainAnalogInputStartAddress    : Main alaog input   | */
/*|                                                         header address     | */
/*|                     pw_LVDTHeaderStartAddress         : LVDT header address| */
/*|                     pw_TimerCounterHeaderStartAddress : Timer counter start| */
/*|                                                         address            | */
/*+----------------------------------------------------------------------------+ */
/*| Return Value      : -                                                      | */
/*+----------------------------------------------------------------------------+ */
VOID    v_UpdateHeader_GetHeaderInfos   (WORD* pw_HeaderData,
                                         WORD* pw_GlobalHeaderSizeAddress,
                                         WORD* pw_MainAnalogInputStartAddress,
                                         WORD* pw_LVDTHeaderStartAddress,
                                         WORD* pw_TimerCounterHeaderStartAddress)
        {
        *pw_GlobalHeaderSizeAddress          = 8 / 2;
        *pw_MainAnalogInputStartAddress      = (pw_HeaderData [11] / 2);
        *pw_LVDTHeaderStartAddress           = *pw_MainAnalogInputStartAddress + 2 +
                                               ((pw_HeaderData [*pw_MainAnalogInputStartAddress + 2] >> 4) / 2) +
                                               ((pw_HeaderData [*pw_MainAnalogInputStartAddress + 2 + (pw_HeaderData [*pw_MainAnalogInputStartAddress + 2] >> 4) / 2] >> 4) / 2);
        *pw_TimerCounterHeaderStartAddress   = (pw_HeaderData [13] / 2);

        }
                
/*i_UpdateHeader_GetLVDT*/
/*+----------------------------------------------------------------------------+*/
/*| Function Name     : VOID v_UpdateHeader_GetLVDT                            |*/
/*|                  (WORD*                            pw_Header,              |*/
/*|                   WORD*                            pw_NumberOfLVDTType,    |*/
/*|                   pstr_LVDTSingleInformations      ps_LVDTInformations)    |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Get the all LVDT                                       |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : pw_Header : Header                                     |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : pw_NumberOfLVDTType  : Number of LVDT type             |*/
/*|                     ps_LVDTInformations  : LVDT type informations          |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      :  0: No error                                           |*/
/*|                     -1: Error                                              |*/
/*|                      1: Warning                                            |*/
/*+----------------------------------------------------------------------------+*/
VOID v_UpdateHeader_GetLVDT (WORD*                        pw_Header,
                             WORD*                        pw_NumberOfLVDTType,
                             pstr_LVDTSingleInformations  ps_LVDTInformations)
        {
        /************************/
        /* Variable declaration */
        /************************/
        WORD w_GlobalHeaderSizeAddress        = 0;
        WORD w_MainAnalogInputStartAddress    = 0;
        WORD w_LVDTHeaderStartAddress         = 0;
        WORD w_TimerCounterHeaderStartAddress = 0;
        
        /***********************************/
        /* Get the old header informations */
        /***********************************/
        v_UpdateHeader_GetHeaderInfos   (pw_Header,
                                         &w_GlobalHeaderSizeAddress,
                                         &w_MainAnalogInputStartAddress,
                                         &w_LVDTHeaderStartAddress,
                                         &w_TimerCounterHeaderStartAddress);
        
        /**************************************/
        /* Get the number of LVDT from header */
        /**************************************/
        *pw_NumberOfLVDTType = pw_Header[w_LVDTHeaderStartAddress + 4] & 0xFF;
        
        /**********************/
        /* Copy all LVDT type */
        /**********************/
        memcpy (ps_LVDTInformations,
                &pw_Header [w_LVDTHeaderStartAddress + 5], 
                sizeof (str_LVDTSingleInformations) * *pw_NumberOfLVDTType);

        }        
