/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.
        
        ADDI-DATA GmbH
        Dieselstrasse 3 
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. 
        
    You shoud find the complete GPL in the COPYING file accompanying 
    this source code. 
* @endverbatim
*/

#include "apci3701-private.h"
#include "vtable.h"
#include "knowndev.h"

MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("APCI-3701 IOCTL driver");

EXPORT_SYMBOL(apci3701_get_lock);

EXPORT_NO_SYMBOLS;

//----------------------------------------------------------------------------
spinlock_t * apci3701_get_lock(struct pci_dev *pdev)
{
	return &(APCI3701_PRIVDATA(pdev)->lock);
}
//------------------------------------------------------------------------------
atomic_t apci3701_count = ATOMIC_INIT(0);
unsigned int apci3701_majornumber = 0;

//------------------------------------------------------------------------------
static int __devinit apci3701_probe_one(struct pci_dev *dev,const struct pci_device_id *ent);
static void __devexit apci3701_remove_one(struct pci_dev *dev);					     

//------------------------------------------------------------------------------
/** The ID table is an array of struct pci_device_id ending with a all-zero entry.
Each entry consists of:

	- vendor, device	Vendor and device ID to match (or PCI_ANY_ID)
	- subvendor,	Subsystem vendor and device ID to match (or PCI_ANY_ID)
	- subdevice class,		Device class to match. The class_mask tells which bits
	- class_mask	of the class are honored during the comparison.
	- driver_data	Data private to the driver.
*/

static struct pci_device_id apci3701_pci_tbl[] __initdata = {
	{
		apci3701_BOARD_VENDOR_ID,
		apci3701_BOARD_DEVICE_ID,
		PCI_ANY_ID,
		PCI_ANY_ID,
		0,
		0,
		0
	},

	{
		apci3701_BOARD_VENDOR_ID,
		apci3702_BOARD_DEVICE_ID,
		PCI_ANY_ID,
		PCI_ANY_ID,
		0,
		0,
		0
	},
	
	{ 0, },	/* 0 terminated list. */
};

MODULE_DEVICE_TABLE (pci, apci3701_pci_tbl);

//----------------------------------------------------------------------------------------------------------------------
static struct pci_driver apci3701_pci_driver = {
	.name		= __DRIVER_NAME,
	.probe		= apci3701_probe_one,
	.remove		= __devexit_p(apci3701_remove_one),
	.id_table	= apci3701_pci_tbl,
};
//------------------------------------------------------------------------------
static struct file_operations apci3701_fops =
{
	.ioctl		= apci3701_ioctl_lookup,
	.open		= apci3701_open_lookup,
	.release	= apci3701_release_lookup,
	.read 		= apci3701_read,
 /*
  * If using signal to inform user mode,
  * uncomment following line:
  */  
  //.fasync		= apci3701_fasync_lookup,  
};

//------------------------------------------------------------------------------
/** when module is unloaded, stop all board activities (cf interrupt)*/
static void apci3701_stop_board(struct pci_dev * pdev)
{
	// FILL THE BLANK
}

//-------------------------------------------------------------------
/** event: a new card is detected.
 * 
 * Historically - in former ADDI-DATA drivers - data about a board has been stored 
 * in a static structure.
 * This led to huge duplication of information since most of these information 
 * are already present in the pci_dev struct. 
 * 
 * Now we manipulate this standard structure provided by the OS and we attach 
 * private data using the field driver_data, if necessary. 
 * 
 *  */
static int __devinit apci3701_probe_one(struct pci_dev *dev, const struct pci_device_id *ent)
{
	{
		int ret = pci_enable_device(dev);
		if (ret)
		{
			printk(KERN_ERR "%s: pci_enable_device failed\n",__DRIVER_NAME);
			return ret;
		}       
    }

	// needed to enable PCI DMA 
	pci_set_master(dev);
	
	/* 2.4 only : check_region is deprecated in 2.6 */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)	
		/* check if BAR 0 is free */
		if ( check_region(dev->resource[0].start, pci_resource_len(dev,0)) ) 
		{
			printk(KERN_WARNING "%s: BAR 0 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[0].start);
			return -EBUSY;
		}
		/* check if BAR 1 is free */
		if ( check_region(dev->resource[1].start, pci_resource_len(dev,1)) ) 
		{
			printk(KERN_WARNING "%s: BAR 1 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[1].start);
			return -EBUSY;
		}
		/* check if BAR 2 is free */
		if ( check_region(dev->resource[2].start, pci_resource_len(dev,2)) ) 
		{
			printk(KERN_WARNING "%s: BAR 2 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[2].start);
			return -EBUSY;
		}
		/* check if BAR 3 is free */
		if ( check_region(dev->resource[3].start, pci_resource_len(dev,3)) ) 
		{
			printk(KERN_WARNING "%s: BAR 3 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[3].start);
			return -EBUSY;
		}
	#endif // 2.4

	/* allocate a new data structure containing board private data */
	{
		struct apci3701_str_BoardInformations * newboard_data = NULL;
		newboard_data = kmalloc( sizeof( struct apci3701_str_BoardInformations) , GFP_ATOMIC);
		if (!newboard_data)
		{
			printk(KERN_CRIT "Can't allocate memory for new board %s\n",pci_name(dev));
			return -ENOMEM;
		}
		/* link standard data structure to the board's private one */	
		//dev->driver_data = newboard_data;
		pci_set_drvdata(dev,newboard_data);
		
		apci3701_init_priv_data(newboard_data);				
	}	

	/* lock BAR IO ports ressources */
	{
	    int ret = pci_request_regions(dev,__DRIVER_NAME);
	    if (ret)
	    {
	        printk(KERN_ERR "%s: pci_request_regions failed\n",__DRIVER_NAME);
	        /* free all allocated ressources here*/
	        kfree(APCI3701_PRIVDATA(dev));
	        return ret;
	    }
	}
	
	/* Map the memory Address 0*/
	APCI3701_PRIVDATA(dev)->dw_MemBaseAddress0 = ioremap (dev->resource[0].start,pci_resource_len(dev,0));
	
	/* Map the memory Address 3*/	
	APCI3701_PRIVDATA(dev)->dw_MemBaseAddress3 = ioremap (dev->resource[3].start,pci_resource_len(dev,3));	

	/* register interrupt */
	// if ( apci3701_register_interrupt(dev) )
	// {
	// 	/* failed, clean previously allocated resources */
	// 	kfree(APCI3701_PRIVDATA(dev));
	// 	pci_release_regions(dev);		
	// }


	{
		/* increase the global board count */
		atomic_inc(&apci3701_count);		
		printk(KERN_INFO "%s: board %s managed (minor number will be %d) \n",__DRIVER_NAME, pci_name(dev), atomic_read(&apci3701_count)-1);
	}
	
	{
		/* Get the APCI-3701 header */
		int error = 0;
		if ((error = i_APCI3701_GetBoardEeprom (dev)) != 0) 
		{
			printk (KERN_CRIT "Can't get header informations (error = %d)\n", error);
			iounmap(APCI3701_PRIVDATA(dev)->dw_MemBaseAddress3);
			iounmap(APCI3701_PRIVDATA(dev)->dw_MemBaseAddress0);
			pci_release_regions (dev);
			kfree (APCI3701_PRIVDATA (dev));
			return -ENODATA;
		}
//		{
//		uint16_t w_TrasnducerIndex = 0;
//		error = i_APCI3701_GetTransducerIndex (dev,	20, &w_TrasnducerIndex);
//		printk (KERN_CRIT "i_APCI3701_GetTransducerIndex (error = %d) index = %d\n", error, w_TrasnducerIndex);
//		}		
	}
	
	/* create /proc entry */
	apci3701_proc_create_device(dev, atomic_read(&apci3701_count)-1);
	
    /* add to the list of known device */
    apci3701_known_dev_append(dev);	
	
	return 0;
}


//------------------------------------------------------------------------------
/** This function registers the driver with register_chrdev.
 * 
 * @todo For kernel 2.6, use udev
 *  */

static int apci3701_register(void)
{
	
	//Registration of driver 
	apci3701_majornumber = register_chrdev(0, __DRIVER_NAME, &apci3701_fops);
	
	if (apci3701_majornumber < 0) 
	{
		printk (KERN_ERR "%s: register_chrdev returned %d ... aborting\n",__DRIVER_NAME,apci3701_majornumber);
		return -ENODEV;
	}
	return 0;
}

//------------------------------------------------------------------------------
/** Called when module loads. */
static int __init apci3701_init(void)
{

	apci3701_init_vtable(apci3701_vtable);
	
	if( apci3701_register() )
	{
		return -ENODEV;
	}
	
	/* registred, now create root /proc entry */
	apci3701_proc_init();    

	printk(KERN_INFO "%s: loaded\n",__DRIVER_NAME);	
					
	/* now, subscribe to PCI bus subsystem  */	
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
	/** @note The module will load anyway even if the call to pci_module_init() fails */
	if ( pci_module_init (&apci3701_pci_driver) != 0)
#else
	if ( pci_register_driver (&apci3701_pci_driver) != 0)
#endif
	{
		printk("%s: no device found\n",__DRIVER_NAME);
		apci3701_pci_driver.name = "";	
	}
			
	return 0;
}

//-------------------------------------------------------------------	     
/** event: a card is removed (also called when module is unloaded) */
static void __devexit apci3701_remove_one(struct pci_dev *dev)
{
	
	/* stop board activities */
	apci3701_stop_board(dev);
	
	/* register interrupt */
	//apci3701_deregister_interrupt(dev);
	
	/* do OS-dependant thing we don't really want to know of :) */	
	pci_disable_device(dev);
	
    /* Unmap the memory Address 0*/
    iounmap(APCI3701_PRIVDATA(dev)->dw_MemBaseAddress0 );
	
    /* Unmap the memory Address 3*/
    iounmap(APCI3701_PRIVDATA(dev)->dw_MemBaseAddress3 );	
	
	/* deallocate BAR IO Ports ressources */
	pci_release_regions(dev);
	
	/* free the eeprom and header data */
	if (APCI3701_PRIVDATA(dev)->pw_BoardEeprom)
		kfree (APCI3701_PRIVDATA(dev)->pw_BoardEeprom);
	
	/* free private device data*/
	if (APCI3701_PRIVDATA(dev))
		kfree(APCI3701_PRIVDATA(dev));
	
	/* delete associated /proc entry */	
	apci3701_proc_release_device(dev);
	
	/* decrement global board count*/	
	atomic_dec(&apci3701_count);
	
    /* remove from the list of known devices */
    apci3701_known_dev_remove(dev);	
	
	printk("%s: board %s unloaded \n",__DRIVER_NAME,pci_name(dev));
	
}

//------------------------------------------------------------------------------
/** This function unregisters the driver with unregister_chrdev.
 * 
 * @todo For kernel 2.6, use udev
 *  */

static void apci3701_unregister(void)
{
	unregister_chrdev(apci3701_majornumber, __DRIVER_NAME);		
}

//-------------------------------------------------------------------
/** Called when module is unloaded. */
static void __exit apci3701_exit(void)
{
 
	/* unsubscribe to PCI bus subsystem */
	if (apci3701_pci_driver.name[0])  
	{
		pci_unregister_driver (&apci3701_pci_driver);
	}
	
	/* unsubscribe to /dev VFS */
	apci3701_unregister();
	
	/* delete /proc root */
	apci3701_proc_release();
}
//------------------------------------------------------------------------------
module_exit(apci3701_exit);
module_init(apci3701_init);


