/** @file testapci3701.c
*
* @author Sylvain Nahas
*
* This software 
* - scans the /proc/sys/apci3701/ directory 
* - run a serie of test on each detected boards
* 
* IMPORTANT: THIS SOFTWARE TEST ALL AVAILABLE FUNCTIONS OF THE DRIVER.
* 			 IT IS POSSIBLE THAT BY RUNNING SOME ERRORS OCCURED.
* 			 IN ORDER TO TEST THE ERROR MANAGEMENT
* 			 SOME FUNCTIONS ARE CALLED WITH BAD PARAMTERS VALUE.
* 			 THIS TEST PROGRAM IS A DEVELOPER TOOL.
*			 SAMPLEapci3701 DEMONSTRATE HOW TO USE THE DRIVER.
*/

/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.
        
        ADDI-DATA GmbH
        Dieselstrasse 3 
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. 
        
    You shoud find the complete GPL in the COPYING file accompanying 
    this source code. 
* @endverbatim
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <sys/sysmacros.h> /* major() minor() */

#include <apci3701.h>


#define APCI_SPECIAL_FILE_DIR "/proc/sys/apci3701/"

//------------------------------------------------------------------------------
#ifdef CONFIG_APCI_PROC_FS_SIMPLE /* special device file is directly under APCI_SPECIAL_FILE_DIR */
	static const char proc_path_template[] = "%s%s";
#else /* special device file is in a sub-directory */
	static const char proc_path_template[] = "%s%s/ctrl";
#endif
//------------------------------------------------------------------------------
/** Filter the "." and ".." entries in the struct dirent ** area set up by the scandir() function (utility function).*/
int
get_real_dir_entry (const struct dirent * ent)
{
        if (! strcmp(ent->d_name,"."))
                return 0;
        if (! strcmp(ent->d_name,".."))
                return 0;
                                
        return 1;
}

//--------------------------------------------------------------------------------
/** this function look up for cards in the /proc/sys/apci3701/ directory 
* @param cards A pointer to the card area to fill
* @return The number of cards detected (may be zero if nor cards found).
*/
int apci3701_find_cards(int ** cards)
{
	
	struct dirent **eps;
	int cnt = 0; // number of card found
	
	/* scan directory for card entry */
	if ( (cnt = scandir (APCI_SPECIAL_FILE_DIR, &eps, get_real_dir_entry , alphasort)) == -1 )
	{
		perror("scandir");
		return 0;
	}	
	
	if (cnt == 0) return 0;
	
	/* allocate cards area */
	if ( ! ( (*cards) = malloc( cnt * sizeof(int) ) ) )
	{
		perror("malloc");
		free(eps);
		return 0;
	}
	
	/* for each card entry, open it and store file descriptor in cards[] */
	{
		int i;
		for (i=0; i< cnt; i++)
		{
			int fd;
			char name[100];
			
			snprintf(name,sizeof(name),proc_path_template,APCI_SPECIAL_FILE_DIR,eps[i]->d_name);
			
			if ( ( fd = open(name, O_RDWR)) == -1 )
			{
				perror("open");
				exit(1);
			}			
									
			{
				struct stat file_stat;
				if ( fstat(fd,&file_stat) )
				{
					perror("fstat");
					exit(1);
				}			
				if (!S_ISCHR(file_stat.st_mode)) 
				{
					/* should never happen !*/
					printf("file %d is not a character device\n",i);
					exit(1);
				}
				printf("%s, major:%d minor:%d \n",name, major(file_stat.st_rdev),minor(file_stat.st_rdev));
			
				(*cards)[minor(file_stat.st_rdev)] = fd;
			} 
			/* index device file descriptor by the minor number */
			
		}	
		
		
		
	}
		
	return cnt;
	
	
}
//--------------------------------------------------------------------------------
void apci3701_run_test(int fd)
{
	printf("=> CMD_apci3701_CheckAndGetPCISlotNumber\n");
	{	
		// deprecated (the number of boards is now unlimited and the OS gives actually these information)
		int nb = 0;
		char b_SlotArray[CONFIG_apci3701_MAX_BOARD_NBR+1];
		if ( (nb = ioctl(fd,CMD_apci3701_CheckAndGetPCISlotNumber,b_SlotArray)) == 0 )
		{
			printf("no boards found\n"); 
			exit(1);
		}	
		printf("The driver advertise %d board\n", nb);
		
		{
			int i;
			for (i=0; i< nb; i++)
	   			printf ("\t%d: Slot Number                       : %d\n", i, b_SlotArray[i]);		
		}
		
	}
	printf("=> CMD_apci3701_CheckAndGetPCISlotNumber with bad address\n");
	{			
		
		if ( (ioctl(fd,CMD_apci3701_CheckAndGetPCISlotNumber,NULL)) == 0 )
		{
			printf("CMD_apci3701_CheckAndGetPCISlotNumber failed\n");
		}	
		
	}	
	printf("=> CMD_apci3701_SetBoardInformation\n");
	{
		//this command do nothing
		if ( ioctl (fd,CMD_apci3701_SetBoardInformation,0) == -1)
		{
			perror("ioctl");
			exit(1);
		}	
	}
	printf("=> CMD_apci3701_GetHardwareInformation\n");
	{
		// deprecated (the OS gives actually these information)
		str_BaseInformations s_BaseInformations;
		if ( ioctl (fd,CMD_apci3701_GetHardwareInformation,&s_BaseInformations) == -1)
		{
			perror("ioctl");
			exit(1);
		}			

   		printf("\tBAR 0 : %d\n",  s_BaseInformations.ui_BaseAddress[0] ); 
		printf("\tBAR 1 : %d\n",  s_BaseInformations.ui_BaseAddress[1] ); 
		printf("\tBAR 2 : %d\n",  s_BaseInformations.ui_BaseAddress[2] ); 
		printf("\tBAR 3 : %d\n",  s_BaseInformations.ui_BaseAddress[3] ); 
		printf("\tBAR 5 : %d\n",  s_BaseInformations.ui_BaseAddress[4] ); 
   		printf("\tinterrupt nb: %d\n", s_BaseInformations.b_Interrupt);
   		printf("\tslot number: %d\n", s_BaseInformations.b_SlotNumber);
		
		
	}
	printf("=> CMD_apci3701_GetHardwareInformation with bad address\n");
	{
		if ( ioctl (fd,CMD_apci3701_GetHardwareInformation,NULL) )
		{
			perror("ioctl");
		}			
		
	}	

}

//--------------------------------------------------------------------------------
void about_test (void)
{
	printf ("IMPORTANT:\n");
	printf ("\tTHIS SOFTWARE TEST ALL AVAILABLE FUNCTIONS OF THE DRIVER.\n");
	printf ("\tIT IS POSSIBLE THAT BY RUNNING SOME ERRORS OCCURED.\n");
	printf ("\tIN ORDER TO TEST THE ERROR MANAGEMENT\n");
	printf ("\tSOME FUNCTIONS ARE CALLED WITH BAD PARAMTERS VALUE.\n");
	printf ("\tTHIS TEST PROGRAM IS A DEVELOPER TOOL.\n");
	printf ("\tSAMPLEapci3701 DEMONSTRATE HOW TO USE THE DRIVER.\n");
}

//--------------------------------------------------------------------------------
int main(int argc, char** argv)
{

	int * apci3701_card_fd;
	unsigned int apci3701_card_number = 0;
	
	about_test ();
	
	{
		apci3701_card_number = apci3701_find_cards(&apci3701_card_fd);
		
		printf("total: %d cards\n",apci3701_card_number);
	}	
	
	{
		int i;
		for (i=0;i< apci3701_card_number; i++)
		{
			printf("\n+++++++++++++++++++++++++++++++++++++\nrunning test on device of minor number %d\n+++++++++++++++++++++++++++++++++++++\n\n\n",i);
			apci3701_run_test(apci3701_card_fd[i]);		
		}
	}

	

	return 0;
}
//--------------------------------------------------------------------------------
